// begin enum.h

/*
	Copyright (c) 2003 Mitch T. Besser, All rights reserved.	

	Permission to use, modify, copy, sell, and distribute this software 
	and its documentation for any purpose is hereby granted without fee, 
	provided that the above copyright notice appears in all copies and 
	that both that copyright notice and this permission notice appear 
	in relevant supporting documentation.  Mitch T. Besser makes no 
	representations about the suitability of this software for any purpose.  
	It is provided "as is" without express or implied warranty.  
*/

/*
	Last Updated:  20-FEB-2003 

	Written by:  Mitch T. Besser <bessermt@yahoo.com> 

	Title:  Generic Printable ENUM++ 

	The article describing this code was published in the C/C++ User's Journal.  

	Thanks to Ilya Ratner, Michelle Franz, Laura Freeman for reviewing the 
	article.  A special thanks to my wife Diane for her support.  

	ENUM++ has only been tested on the Microsoft C++ 6.0 compiler.  While ENUM++ 
	should not contain any Microsoft specific code, I can not promise it will 
	work in your environment.  Please contact me if you find any 
	incompatiblities with your environment.  I will gladly make any corrections 
	required.  

	Feel free to contact me if you have any questions, comments or suggestions.  


	File Description:  

	This file contains the ENUM++ macros and templates which generate enum like 
	data types without additional overhead in speed or size and with additional 
	abilities beyond the standard C++ enum, including:  

	- Conversion of values to formatted and unformatted text.  
	- Optional std::istream and std::ostream support.  
	- STL style iteration, including begin() and end().  
	- Generation of the number of contained elements.  
	- Detection of uninitialized and out of range values.  
	- Standardization of the uninitialized state.  
	- Compile-time configurable behavior.  


	You may optionally do the following manually at global scope:  

	DECLARE_ENUM_ISTREAM(type); // input function declaration 
	DECLARE_ENUM_OSTREAM(type); // output function declaration 
	DEFINE_ENUM_ISTREAM(type);  // definition to allow formatted stream input 
	DEFINE_ENUM_OSTREAM(type);  // definition to allow formatted stream output 

	Typically, you will add the DECLARE_ENUM_OSTREAM and DECLARE_ENUM_ISTREAM 
	macro(s) immediately after the ENUM++ declaration in a header file.  Then 
	you add the DEFINE_ENUM_OSTREAM and DEFINE_ENUM_ISTREAM macros in one source 
	file.  ENUM++ ostreams and istreams pay attention to the width manipulator.  

	You do not need to use any special value to denote the uninitialized state.  
	The ENUM++ macros support this automatically.  While you may define your 
	own uninitialized state, I encourage you to let ENUM++ do this for you.  

	The ENUM++ macros do not allow you to explicitly initialize enumeration 
	constants.  You must allow the macro to choose the values for you.  If you 
	really need this feature, feel free to contact me.  I've often thought this 
	would be a useful and difficult to implement feature.  

	//  Bad code // ENUM4(Color, Uninitialized=-1, Red, Green=7, Blue);
	// Good code // ENUM3(Color, Red, Green, Blue);

	The current limit on ENUM++ is ENUM50.  If you have enumerations with more 
	than 50 values, you will need to add more macros to handle the new limit.  

	Member functions (implicit and explicit):  

	type();                      // ctor initializing to default (ENUM_DEFAULT_VALUE) 
	type(const type&);           // copy constructor (using default) 
	type(const value_type);      // ctor initializing to scoped value 
	type(const _enum_type);      // ctor initializing to unscoped value 
	explicit type(const int);    // initialize with an int (not recommended) 
	operator=(const type&);      // copy assignment (using default) 
	operator=();                 // allows you to assign, but not with an integer 
	text();                      // converts variable's value into a formatted string 
	unformatted_text();          // converts variable's value into an unformatted string 
	static begin();              // returns the first constant value 
	static end();                // returns one beyond the last constant value 
	static size();               // the number of constants in the enumeration (aka SIZE)
	static default_value();      // returns the default value (aka DEFAULT_VALUE)
	static invalid_value();      // returns the invalid value (aka INVALID_VALUE)
	operator++();                // pre and postfix increment to the next constant value 
	operator--();                // pre and postfix decrement to the previous constant value 
	clear();                     // resets the value to default state (ENUM_DEFAULT_VALUE) 
	invalidate();                // resets the value to the invalid state (ENUM_INVALID_VALUE) 
	is_valid() const;            // tests value is within the valid range (this is the preferred test) 
	is_default() const;          // tests value is set to the default value 
	is_initialized() const;      // tests value is not set to an uninitialized state 
	assign(const int);           // forced assignment of an int (not recommended) 
	assign(const StringType&);   // assignment of formatted text 
	operator StringType() const; // implicit conversion to string (non-strict only method) 
	operator int() const;        // implicit and explicit conversion to int 

	Random access iterator like methods.  For further details see: 
	<http://www.sgi.com/tech/stl/RandomAccessIterator.html>

	operator+=(const difference_type);     // forward motion 
	operator+(const difference_type rhs);  // iterator addition 
	operator+(const EnumSub rhs);          // iterator addition 
	operator-=(const difference_type rhs); // backward motion 
	operator-(const difference_type rhs);  // iterator subtraction 
	operator-(const EnumSub rhs);          // iterator subtraction 


	For actual member function signatures, see the implementation below.  

	Identifiers containing an underscore (_) prefix are for internal 
	use only and are especially subject to change without notice.  


	If you need to configure enum.h, you may define any combination of the 
	following preprocessor values as needed:  

	ENUM_ASSERT              // Defines an alternate assertion statement 
	ENUM_STRICT              // Only allow a restrictive set of functionality 
	ENUM_INVALID_VALUE       // Internal invalid value (default==-1) 
	ENUM_DEFAULT_VALUE       // Internal default value (default==ENUM_INVALID_VALUE) 
	ENUM_LOCAL_ONLY          // Enforce scoped constants 
	ENUM_NO_DEFAULT          // Remove default initialization safety feature 
	ENUM_TCHAR               // Character type (default==char) 
	ENUM__T                  // Character type conversion macro 
	ENUM_SPACE_SUB           // Formatter's space substitution character (default=='_') 
	ENUM_TEXT_FORMAT_FUNCTOR // Name of text formatting functor 
	ENUM_COMPARE             // Make comparison functions available (experimental) 

	Example:  
	If you use MFC, you may wish to add 
	/D ENUM_ASSERT=ASSERT 
	to your project settings in order to use the MFC supplied ASSERT 
	macro instead of the standard C++ assert() method.  

	// example code:  

	ENUM12(Month, Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec);

	DEFINE_ENUM_OSTREAM(Month) // defines operator<<() for type Month 

	void f()
	{
		Month m=Month::Dec;
		std::cout << "Pearl Harbor - " << m << " 7th, 1941" << std::endl;
	}
*/

#ifdef TRACE_INCLUDE
#	ifndef TRACE_INCLUDE_MSG
#		define TRACE_INCLUDE_MSG "#included "
#	endif
#	if _MSC_VER>=1200
#		define STRINGIZE_LINE_DELAY(L) #L
#		define STRINGIZE_LINE(L) STRINGIZE_LINE_DELAY(L)
#		pragma message(__FILE__ "(" STRINGIZE_LINE(__LINE__) ") : info: " TRACE_INCLUDE_MSG)
#		undef STRINGIZE_LINE
#		undef STRINGIZE_LINE_DELAY
#	else
#		error insert your compiler's trace here.  
#	endif
#endif

#ifndef _KEA_ENUM_H_
#define _KEA_ENUM_H_

// Begin includes
#include <cassert>
#include <cctype>
#include <cstddef>
#include <cstring>
#include <string>
#include <functional>
#include <algorithm>
#include <vector>
#ifdef _MSC_VER
#	include <tchar.h>
#endif
#include <boost/preprocessor/tuple/to_list.hpp> // http://www.boost.org/
#include <boost/preprocessor/list/enum.hpp> // http://www.boost.org/
#include "StreamUDT.h"
// End includes

#ifndef ENUM_TCHAR
#	ifdef TCHAR
#		define ENUM_TCHAR TCHAR
#	else
#		define ENUM_TCHAR char
#	endif
#endif

#ifndef ENUM__T
#	ifdef _T
#		define ENUM__T(x) _T(x)
#	else
#		define ENUM__T(x) x
#	endif
#endif

#ifndef ENUM_ASSERT
#	ifdef RTASSERT
#		define ENUM_ASSERT(expression) RTASSERT(expression)
#	else
#		define ENUM_ASSERT(expression) assert(expression)
#	endif
#endif

#ifndef ENUM_SPACE_SUB
#	define ENUM_SPACE_SUB '_'
#endif

#ifndef ENUM_INVALID_VALUE
#	define ENUM_INVALID_VALUE -1
#endif

#ifndef ENUM_DEFAULT_VALUE
#	define ENUM_DEFAULT_VALUE ENUM_INVALID_VALUE
#endif

#ifndef ENUM_TEXT_FORMAT_FUNCTOR
#	define ENUM_TEXT_FORMAT_FUNCTOR Format
#endif

#ifndef ENUM_LOCAL_ONLY
#	define ENUM_GLOBAL_VALUES(type, value_list) enum _##type {value_list};
#else
#	define ENUM_GLOBAL_VALUES(type, value_list) enum _##type {};
#endif

namespace KEA
{
	template<class EnumSub, size_t N, class StringType>
	class _EnumText
	{
	private:
		typedef StringType::value_type value_type;

		StringType text_map_[N];
		StringType* best_match_order_[N];

		struct Format: public std::unary_function<const char [], StringType>
		{
			result_type operator()(argument_type unformatted) const
			{
				StringType formatted;
				const char* current=unformatted;
				if (strchr(unformatted, ENUM_SPACE_SUB)!=0)
				{
					while (*current)
					{
						char c=*current;
						if (c==ENUM_SPACE_SUB)
						{
							c=' ';
						}
						formatted+=value_type(c);
						++current;
					}
				}
				else
				{
					while (*current)
					{
						while (islower(*current))
						{
							formatted+=value_type(*current);
							++current;
						}
						if (!*current)
						{
							break;
						}
						if (current!=unformatted)
						{
							formatted+=value_type(' ');
						}
						if (isdigit(*current))
						{
							do
							{
								formatted+=value_type(*current);
								++current;
							} while (isdigit(*current));
							if (islower(*current))
							{
								formatted+=value_type(' ');
							}
						}
						else
						{
							const char* const first=current;
							const char* last;
							for (;;)
							{
								last=current;
								++current;
								if (!isupper(*current))
								{
									break;
								}
								formatted+=value_type(*last);
							}
							if (*current && last!=first)
							{
								formatted+=value_type(' ');
							}
							formatted+=value_type(*last);
						}
					}
				}
				return formatted;
			}
		};

		struct StringLengthGreater: public std::binary_function<const StringType*, const StringType*, bool>
		{
			result_type operator()(first_argument_type first_argument, second_argument_type second_argument) const
			{
				const StringType::size_type first_argument_length=first_argument->length();
				const StringType::size_type second_argument_length=second_argument->length();
				return first_argument_length>second_argument_length;
			}
		};

		void sort_best_match()
		{
			StringType** const first=best_match_order_;
			const StringType* const * const last=first+N;

			StringType* text_map=text_map_;

			for (StringType** current=first; current!=last; ++current)
			{
				*current=text_map++;
			}

			std::sort(best_match_order_, best_match_order_+N, StringLengthGreater());
		}

	private: // intentionally not implemented:
		_EnumText(const _EnumText&);
		_EnumText& operator=(const _EnumText&);

	private:
		_EnumText()
		{
			const char* const * const first=EnumSub::_get_map();
			const char* const * const last=first+N;

			std::transform(first, last, text_map_, ENUM_TEXT_FORMAT_FUNCTOR());
			sort_best_match();
		}

	public:
		static inline _EnumText& Instance()
		{
			static _EnumText enum_text;
			return enum_text;
		}

		inline const StringType* const* get_best_match_order() const
		{
			return best_match_order_;
		}

		inline StringType::size_type max_length() const
		{
			static const StringType::size_type length=N>0?best_match_order_[0]->length():0;
			return length;
		}

		inline const StringType& get(const size_t index) const
		{
			return text_map_[index];
		}
	};

	template<typename _enumType, class EnumSub, size_t N>
	class _EnumSuper
	{
	public:
		typedef size_t size_type;
		typedef int difference_type;

	private:
		typedef ENUM_TCHAR char_type;
		typedef std::basic_string<char_type> StringType;

	protected:
		typedef _enumType _enum_type;

	public:
		enum
		{
			// Less preferred alternatives for...
			INVALID_VALUE=ENUM_INVALID_VALUE, // invalid_value()
			DEFAULT_VALUE=ENUM_DEFAULT_VALUE, // default_value()
			SIZE=N                            // size()
		};

	private:
		_enum_type value_;

	private:
		static inline const _EnumText<EnumSub, N, StringType>& get_enum_text()
		{
			_EnumText<EnumSub, N, StringType>& enum_text=
				_EnumText<EnumSub, N, StringType>::Instance();
			return enum_text;
		}

	protected: // constructors:  
		inline _EnumSuper() 
#ifndef ENUM_NO_DEFAULT
		:value_(_enum_type(DEFAULT_VALUE))
#endif
		{
		}

		inline explicit _EnumSuper(const int value): 
			value_(_enum_type(value))
		{
			ENUM_ASSERT(is_initialized());
		}

		inline explicit _EnumSuper(const _enum_type value): 
			value_(value)
		{
			ENUM_ASSERT(is_initialized());
		}

	private:
		inline bool in_range() const
		{
			const bool result=value_>=0 && value_<=N;
			ENUM_ASSERT(result || value_==_enum_type(INVALID_VALUE) || value_==_enum_type(DEFAULT_VALUE));
			return result;
		}

	protected:
		inline _enum_type _get_value() const
		{
			ENUM_ASSERT(in_range());
			return value_;
		}

	public:
		inline operator _enum_type() const
		{
			return _get_value();
		}

#ifndef ENUM_STRICT
		inline operator StringType() const
		{
			return text();
		}
#endif

//	intentionally not implemented:
//		operator const char*() const;

	public:
		inline bool is_initialized() const
		{
			const bool result=value_!=_enum_type(INVALID_VALUE);
			return result;
		}

		inline bool is_default() const
		{
			const bool result=value_==_enum_type(DEFAULT_VALUE);
			return result;
		}

		inline bool is_valid() const
		{
			const bool result=is_initialized() && value_>=0 && value_<N;
			return result;
		}

		inline void clear()
		{
			value_=_enum_type(DEFAULT_VALUE);
		}

		inline void invalidate()
		{
			value_=_enum_type(INVALID_VALUE);
		}

		inline static EnumSub begin()
		{
			return _enum_type(0);
		}

		inline static EnumSub end()
		{
			return _enum_type(N);
		}

		inline static size_type size()
		{
			return SIZE;
		}

		inline static EnumSub default_value()
		{
			return _enum_type(DEFAULT_VALUE);
		}

		inline static EnumSub invalid_value()
		{
			return _enum_type(INVALID_VALUE);
		}

	public:
		inline void assign(const int value)
		{
			value_=_enum_type(value);
			ENUM_ASSERT(is_initialized());
		}

		inline void assign(const _enum_type value)
		{
			value_=value;
			ENUM_ASSERT(is_initialized());
		}

		inline void assign(const EnumSub value)
		{
			value_=value;
			ENUM_ASSERT(is_initialized());
		}

		inline void assign(const StringType& formatted_text)
		{
			clear();
			const EnumSub last=end();
			for (EnumSub current=begin(); current!=last; ++current)
			{
				if (current.text()==formatted_text)
				{
					value_=current;
					break;
				}
			}
			ENUM_ASSERT(is_valid());
		}

	protected:
		void advance(const difference_type delta)
		{
			value_=_enum_type(value_+delta);
			ENUM_ASSERT(in_range());
		}

#ifdef ENUM_COMPARE // (not implemented yet)
	private:
// Place disallowed comparisons here.  

	public:
		inline bool operator!() const
		{
			const bool result=!_get_value();
			return result;
		}

		template<class T>
		inline bool operator==(const T& rhs) const
		{
			const bool result=_get_value()==rhs;
			return result;
		}

		template<class T>
		inline bool operator!=(const T& rhs) const
		{
			const bool result=!(*this==rhs);
			return result;
		}

		template<class T>
		inline bool operator<(const T& rhs) const
		{
			const bool result=_get_value()<rhs;
			return result;
		}

		template<class T>
		inline bool operator<=(const T& rhs) const
		{
			const bool result=!(rhs<*this);
			return result;
		}

		template<class T>
		inline bool operator>(const T& rhs) const
		{
			const bool result=rhs<*this;
			return result;
		}

		template<class T>
		inline bool operator>=(const T& rhs) const
		{
			const bool result=!(*this<rhs);
			return result;
		}
#endif

	public:
#ifndef ENUM_STRICT
		inline EnumSub operator*() const 
		{
			const EnumSub result=_get_value();
			return result;
		}

		inline EnumSub& operator*()
		{
			ENUM_ASSERT(in_range());
			return static_cast<EnumSub&>(*this);
		}
#endif

		inline EnumSub& operator++()
		{
			advance(+1);
			return static_cast<EnumSub&>(*this);
		}

		inline EnumSub& operator++(int)
		{
			return ++(*this);
		}

		inline EnumSub& operator--()
		{
			advance(-1);
			return static_cast<EnumSub&>(*this);
		}

		inline EnumSub& operator--(int)
		{
			return --(*this);
		}

		inline EnumSub& operator+=(const difference_type rhs)
		{
			advance(rhs);
			return static_cast<EnumSub&>(*this);
		}

		inline EnumSub operator+(const difference_type rhs) const
		{
			EnumSub value=_get_value();
			value.advance(rhs);
			return value;
		}

		inline EnumSub operator+(const EnumSub rhs) const
		{
			const EnumSub value=*this+rhs._get_value();
			return value;
		}

		friend inline EnumSub operator+(const difference_type lhs, EnumSub rhs)
		{
			rhs=rhs+lhs;
			return rhs;
		}

		inline EnumSub& operator-=(const difference_type rhs)
		{
			advance(-rhs);
			return static_cast<EnumSub&>(*this);
		}

		inline EnumSub operator-(const difference_type rhs) const
		{
			EnumSub value=_get_value();
			value.advance(-rhs);
			return value;
		}

		inline EnumSub operator-(const EnumSub rhs) const
		{
			const EnumSub value=*this-rhs._get_value();
			return value;
		}

		friend inline difference_type operator-(const difference_type lhs, const EnumSub rhs)
		{
			const difference_type result=lhs-rhs._get_value();
			return result;
		}

	public:
		StringType unformatted_text() const;
		const StringType& text() const;

	private:
		class StringPtrLenMatch: public std::unary_function<const StringType* const, bool>
		{
		private: // intentionally not implemented:
			StringPtrLenMatch(const StringPtrLenMatch&);
			StringPtrLenMatch& operator=(const StringPtrLenMatch&);

		private:
			const StringType value_;

		public:
			StringPtrLenMatch(StringType::const_reference value):
			value_(&value)
			{
			}

			result_type operator()(argument_type string_ptr) const
			{
				ENUM_ASSERT(string_ptr!=0);
				const StringType::size_type length=string_ptr->length();
				const result_type match=value_.compare(0, length, *string_ptr)==0;
				return match;
			}
		};

	public:
		template<class charT, class Traits>
		std::ios_base::iostate put_stream(std::basic_ostream<charT, Traits>& os) const
		{
			os << text();
			return os.rdstate();
		}

		template<class charT, class Traits>
		std::ios_base::iostate get_stream(std::basic_istream<charT, Traits>& is)
		{
			const std::basic_istream<charT, Traits>::pos_type start_position=is.tellg();
			const _EnumText<EnumSub, N, StringType>& enum_text=get_enum_text();
			const StringType::size_type width=is.width();
			const StringType::size_type length=width==0?enum_text.max_length():width;
			std::vector<charT> value(length+1, 0);
			is.read(&value.front(), length);
			is.clear(is.rdstate() & ~(std::ios_base::eofbit | std::ios_base::failbit));
			is.seekg(start_position);
			const StringType* const* const first=enum_text.get_best_match_order();
			const StringType* const* const last=first+N;
			const StringType* const* const match=
				std::find_if(first, last, StringPtrLenMatch(value.front()));
			if (match==last)
			{
				is.setstate(std::ios_base::failbit);
			}
			else
			{
				const StringType& match_string=**match;
				is.ignore(match_string.length());
				value_=_enum_type(std::distance(&enum_text.get(0), &match_string));
			}
			return is.rdstate();
		}
	};

	template<typename _enumType, class EnumSub, size_t N> 
		_EnumSuper<_enumType, EnumSub, N>::StringType 
		_EnumSuper<_enumType, EnumSub, N>::unformatted_text() const
	{
		StringType result;
		if (is_valid())
		{
			result=*(EnumSub::_get_map()+_get_value());
		}
		else
		{
			result=EnumSub::_get_unknown_text();
			ENUM_ASSERT(false);
		}
		return result;
	}

	template<typename _enumType, class EnumSub, size_t N> 
		const _EnumSuper<_enumType, EnumSub, N>::StringType& 
		_EnumSuper<_enumType, EnumSub, N>::text() const
	{
		const StringType* result;
		if (is_valid())
		{
			const _EnumText<EnumSub, N, StringType>& enum_text=get_enum_text();
			result=&enum_text.get(_get_value());
		}
		else
		{
			result=&EnumSub::_get_unknown_text();
			ENUM_ASSERT(false);
		}
		return *result;
	}
}

#define BEGIN_ENUM(type, N, value_list)              \
                                                     \
ENUM_GLOBAL_VALUES(type, value_list)                 \
                                                     \
class type: public KEA::_EnumSuper<_##type, type, N> \
{                                                    \
public:                                              \
    enum value_type {value_list};                    \
                                                     \
    type()                                           \
    {}                                               \
                                                     \
    explicit type(const int value):                  \
        ::KEA::_EnumSuper<_enum_type, type, N>       \
        (value)                                      \
    {}                                               \
                                                     \
    type(const _enum_type value):                    \
        ::KEA::_EnumSuper<_enum_type, type, N>       \
        (value)                                      \
    {}                                               \
                                                     \
    type(const value_type value):                    \
        ::KEA::_EnumSuper<_enum_type, type, N>       \
        (_enum_type(value))                          \
    {}                                               \
                                                     \
    inline type operator=(const _enum_type value)    \
    {                                                \
        assign(value);                               \
        return *this;                                \
    }                                                \
                                                     \
    inline type operator=(const value_type value)    \
    {                                                \
        assign(value);                               \
        return *this;                                \
    }                                                \
                                                     \
    static const StringType& _get_unknown_text()     \
    {                                                \
        static const StringType unknown_text=        \
            ENUM__T("Unknown "                       \
            ENUM__T(#type));                         \
        return unknown_text;                         \
    }                                                \
                                                     \
    BEGIN_ENUM_MAP

#define BEGIN_ENUM_MAP                        \
        static const char* const * _get_map() \
        {                                     \
            static const char* map[]=         \
            {

#define ENUM_ENTRY(value)         \
                {(value, #value)}

#define END_ENUM_MAP    \
            };          \
            return map; \
        }

#define END_ENUM \
    END_ENUM_MAP \
    }

#undef ENUM_TCHAR
#undef ENUM_ASSERT
#undef ENUM_SPACE_SUB
#undef ENUM_INVALID_VALUE
#undef ENUM_DEFAULT_VALUE
#undef ENUM_TEXT_FORMAT_FUNCTOR

#define DECLARE_ENUM_ISTREAM(type) DECLARE_STREAM_EXTRACTOR(type)

#define DEFINE_ENUM_ISTREAM(type) DEFINE_STREAM_EXTRACTOR(type)

#define DECLARE_ENUM_OSTREAM(type) DECLARE_STREAM_INSERTER(type)

#define DEFINE_ENUM_OSTREAM(type) DEFINE_STREAM_INSERTER(type)

#ifdef ENUM_STRICT
#	define ENUM0(type) type // ENUM0 is probably not very useful.  
#else

#define ENUM0(type)                   \
    BEGIN_ENUM(type, 0, BOOST_PP_NIL) \
        {0}                           \
    END_ENUM
#endif

#define ENUM_LIST(size, tuple) \
BOOST_PP_LIST_ENUM(BOOST_PP_TUPLE_TO_LIST(size, tuple))

#define ENUM1(type, v0)                     \
    BEGIN_ENUM(type, 1, ENUM_LIST(1, (v0))) \
        ENUM_ENTRY(v0)                      \
    END_ENUM

#define ENUM2(type, v0, v1)                     \
    BEGIN_ENUM(type, 2, ENUM_LIST(2, (v0, v1))) \
        ENUM_ENTRY(v0),                         \
        ENUM_ENTRY(v1)                          \
    END_ENUM

#define ENUM3(type, v0, v1, v2)                     \
    BEGIN_ENUM(type, 3, ENUM_LIST(3, (v0, v1, v2))) \
        ENUM_ENTRY(v0),                             \
        ENUM_ENTRY(v1),                             \
        ENUM_ENTRY(v2)                              \
    END_ENUM

#define ENUM4(type, v0, v1, v2, v3)                     \
    BEGIN_ENUM(type, 4, ENUM_LIST(4, (v0, v1, v2, v3))) \
        ENUM_ENTRY(v0),                                 \
        ENUM_ENTRY(v1),                                 \
        ENUM_ENTRY(v2),                                 \
        ENUM_ENTRY(v3)                                  \
    END_ENUM

#define ENUM5(type, v0, v1, v2, v3, v4)                     \
    BEGIN_ENUM(type, 5, ENUM_LIST(5, (v0, v1, v2, v3, v4))) \
        ENUM_ENTRY(v0),                                     \
        ENUM_ENTRY(v1),                                     \
        ENUM_ENTRY(v2),                                     \
        ENUM_ENTRY(v3),                                     \
        ENUM_ENTRY(v4)                                      \
    END_ENUM

#define ENUM6(type, v0, v1, v2, v3, v4, v5)                     \
    BEGIN_ENUM(type, 6, ENUM_LIST(6, (v0, v1, v2, v3, v4, v5))) \
        ENUM_ENTRY(v0),                                         \
        ENUM_ENTRY(v1),                                         \
        ENUM_ENTRY(v2),                                         \
        ENUM_ENTRY(v3),                                         \
        ENUM_ENTRY(v4),                                         \
        ENUM_ENTRY(v5)                                          \
    END_ENUM

#define ENUM7(type, v0, v1, v2, v3, v4, v5, v6)                     \
    BEGIN_ENUM(type, 7, ENUM_LIST(7, (v0, v1, v2, v3, v4, v5, v6))) \
        ENUM_ENTRY(v0),                                             \
        ENUM_ENTRY(v1),                                             \
        ENUM_ENTRY(v2),                                             \
        ENUM_ENTRY(v3),                                             \
        ENUM_ENTRY(v4),                                             \
        ENUM_ENTRY(v5),                                             \
        ENUM_ENTRY(v6)                                              \
    END_ENUM

#define ENUM8(type, v0, v1, v2, v3, v4, v5, v6, v7)                     \
    BEGIN_ENUM(type, 8, ENUM_LIST(8, (v0, v1, v2, v3, v4, v5, v6, v7))) \
        ENUM_ENTRY(v0),                                                 \
        ENUM_ENTRY(v1),                                                 \
        ENUM_ENTRY(v2),                                                 \
        ENUM_ENTRY(v3),                                                 \
        ENUM_ENTRY(v4),                                                 \
        ENUM_ENTRY(v5),                                                 \
        ENUM_ENTRY(v6),                                                 \
        ENUM_ENTRY(v7)                                                  \
    END_ENUM

#define ENUM9(type, v0, v1, v2, v3, v4, v5, v6, v7, v8)                     \
    BEGIN_ENUM(type, 9, ENUM_LIST(9, (v0, v1, v2, v3, v4, v5, v6, v7, v8))) \
        ENUM_ENTRY(v0),                                                     \
        ENUM_ENTRY(v1),                                                     \
        ENUM_ENTRY(v2),                                                     \
        ENUM_ENTRY(v3),                                                     \
        ENUM_ENTRY(v4),                                                     \
        ENUM_ENTRY(v5),                                                     \
        ENUM_ENTRY(v6),                                                     \
        ENUM_ENTRY(v7),                                                     \
        ENUM_ENTRY(v8)                                                      \
    END_ENUM

#define ENUM10(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9)                      \
    BEGIN_ENUM(type, 10, ENUM_LIST(10, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9))) \
        ENUM_ENTRY(v0),                                                           \
        ENUM_ENTRY(v1),                                                           \
        ENUM_ENTRY(v2),                                                           \
        ENUM_ENTRY(v3),                                                           \
        ENUM_ENTRY(v4),                                                           \
        ENUM_ENTRY(v5),                                                           \
        ENUM_ENTRY(v6),                                                           \
        ENUM_ENTRY(v7),                                                           \
        ENUM_ENTRY(v8),                                                           \
        ENUM_ENTRY(v9)                                                            \
    END_ENUM

#define ENUM11(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10)                      \
    BEGIN_ENUM(type, 11, ENUM_LIST(11, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10))) \
        ENUM_ENTRY(v0),                                                                \
        ENUM_ENTRY(v1),                                                                \
        ENUM_ENTRY(v2),                                                                \
        ENUM_ENTRY(v3),                                                                \
        ENUM_ENTRY(v4),                                                                \
        ENUM_ENTRY(v5),                                                                \
        ENUM_ENTRY(v6),                                                                \
        ENUM_ENTRY(v7),                                                                \
        ENUM_ENTRY(v8),                                                                \
        ENUM_ENTRY(v9),                                                                \
        ENUM_ENTRY(v10)                                                                \
    END_ENUM

#define ENUM12(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11)                      \
    BEGIN_ENUM(type, 12, ENUM_LIST(12, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11))) \
        ENUM_ENTRY(v0),                                                                     \
        ENUM_ENTRY(v1),                                                                     \
        ENUM_ENTRY(v2),                                                                     \
        ENUM_ENTRY(v3),                                                                     \
        ENUM_ENTRY(v4),                                                                     \
        ENUM_ENTRY(v5),                                                                     \
        ENUM_ENTRY(v6),                                                                     \
        ENUM_ENTRY(v7),                                                                     \
        ENUM_ENTRY(v8),                                                                     \
        ENUM_ENTRY(v9),                                                                     \
        ENUM_ENTRY(v10),                                                                    \
        ENUM_ENTRY(v11)                                                                     \
    END_ENUM

#define ENUM13(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12)                      \
    BEGIN_ENUM(type, 13, ENUM_LIST(13, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12))) \
        ENUM_ENTRY(v0),                                                                          \
        ENUM_ENTRY(v1),                                                                          \
        ENUM_ENTRY(v2),                                                                          \
        ENUM_ENTRY(v3),                                                                          \
        ENUM_ENTRY(v4),                                                                          \
        ENUM_ENTRY(v5),                                                                          \
        ENUM_ENTRY(v6),                                                                          \
        ENUM_ENTRY(v7),                                                                          \
        ENUM_ENTRY(v8),                                                                          \
        ENUM_ENTRY(v9),                                                                          \
        ENUM_ENTRY(v10),                                                                         \
        ENUM_ENTRY(v11),                                                                         \
        ENUM_ENTRY(v12)                                                                          \
    END_ENUM

#define ENUM14(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13)                      \
    BEGIN_ENUM(type, 14, ENUM_LIST(14, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13))) \
        ENUM_ENTRY(v0),                                                                               \
        ENUM_ENTRY(v1),                                                                               \
        ENUM_ENTRY(v2),                                                                               \
        ENUM_ENTRY(v3),                                                                               \
        ENUM_ENTRY(v4),                                                                               \
        ENUM_ENTRY(v5),                                                                               \
        ENUM_ENTRY(v6),                                                                               \
        ENUM_ENTRY(v7),                                                                               \
        ENUM_ENTRY(v8),                                                                               \
        ENUM_ENTRY(v9),                                                                               \
        ENUM_ENTRY(v10),                                                                              \
        ENUM_ENTRY(v11),                                                                              \
        ENUM_ENTRY(v12),                                                                              \
        ENUM_ENTRY(v13)                                                                               \
    END_ENUM

#define ENUM15(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14)                      \
    BEGIN_ENUM(type, 15, ENUM_LIST(15, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14))) \
        ENUM_ENTRY(v0),                                                                                    \
        ENUM_ENTRY(v1),                                                                                    \
        ENUM_ENTRY(v2),                                                                                    \
        ENUM_ENTRY(v3),                                                                                    \
        ENUM_ENTRY(v4),                                                                                    \
        ENUM_ENTRY(v5),                                                                                    \
        ENUM_ENTRY(v6),                                                                                    \
        ENUM_ENTRY(v7),                                                                                    \
        ENUM_ENTRY(v8),                                                                                    \
        ENUM_ENTRY(v9),                                                                                    \
        ENUM_ENTRY(v10),                                                                                   \
        ENUM_ENTRY(v11),                                                                                   \
        ENUM_ENTRY(v12),                                                                                   \
        ENUM_ENTRY(v13),                                                                                   \
        ENUM_ENTRY(v14)                                                                                    \
    END_ENUM

#define ENUM16(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15)                      \
    BEGIN_ENUM(type, 16, ENUM_LIST(16, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15))) \
        ENUM_ENTRY(v0),                                                                                         \
        ENUM_ENTRY(v1),                                                                                         \
        ENUM_ENTRY(v2),                                                                                         \
        ENUM_ENTRY(v3),                                                                                         \
        ENUM_ENTRY(v4),                                                                                         \
        ENUM_ENTRY(v5),                                                                                         \
        ENUM_ENTRY(v6),                                                                                         \
        ENUM_ENTRY(v7),                                                                                         \
        ENUM_ENTRY(v8),                                                                                         \
        ENUM_ENTRY(v9),                                                                                         \
        ENUM_ENTRY(v10),                                                                                        \
        ENUM_ENTRY(v11),                                                                                        \
        ENUM_ENTRY(v12),                                                                                        \
        ENUM_ENTRY(v13),                                                                                        \
        ENUM_ENTRY(v14),                                                                                        \
        ENUM_ENTRY(v15)                                                                                         \
    END_ENUM

#define ENUM17(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16)                      \
    BEGIN_ENUM(type, 17, ENUM_LIST(17, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16))) \
        ENUM_ENTRY(v0),                                                                                              \
        ENUM_ENTRY(v1),                                                                                              \
        ENUM_ENTRY(v2),                                                                                              \
        ENUM_ENTRY(v3),                                                                                              \
        ENUM_ENTRY(v4),                                                                                              \
        ENUM_ENTRY(v5),                                                                                              \
        ENUM_ENTRY(v6),                                                                                              \
        ENUM_ENTRY(v7),                                                                                              \
        ENUM_ENTRY(v8),                                                                                              \
        ENUM_ENTRY(v9),                                                                                              \
        ENUM_ENTRY(v10),                                                                                             \
        ENUM_ENTRY(v11),                                                                                             \
        ENUM_ENTRY(v12),                                                                                             \
        ENUM_ENTRY(v13),                                                                                             \
        ENUM_ENTRY(v14),                                                                                             \
        ENUM_ENTRY(v15),                                                                                             \
        ENUM_ENTRY(v16)                                                                                              \
    END_ENUM

#define ENUM18(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17)                      \
    BEGIN_ENUM(type, 18, ENUM_LIST(18, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17))) \
        ENUM_ENTRY(v0),                                                                                                   \
        ENUM_ENTRY(v1),                                                                                                   \
        ENUM_ENTRY(v2),                                                                                                   \
        ENUM_ENTRY(v3),                                                                                                   \
        ENUM_ENTRY(v4),                                                                                                   \
        ENUM_ENTRY(v5),                                                                                                   \
        ENUM_ENTRY(v6),                                                                                                   \
        ENUM_ENTRY(v7),                                                                                                   \
        ENUM_ENTRY(v8),                                                                                                   \
        ENUM_ENTRY(v9),                                                                                                   \
        ENUM_ENTRY(v10),                                                                                                  \
        ENUM_ENTRY(v11),                                                                                                  \
        ENUM_ENTRY(v12),                                                                                                  \
        ENUM_ENTRY(v13),                                                                                                  \
        ENUM_ENTRY(v14),                                                                                                  \
        ENUM_ENTRY(v15),                                                                                                  \
        ENUM_ENTRY(v16),                                                                                                  \
        ENUM_ENTRY(v17)                                                                                                   \
    END_ENUM

#define ENUM19(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18)                      \
    BEGIN_ENUM(type, 19, ENUM_LIST(19, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18))) \
        ENUM_ENTRY(v0),                                                                                                        \
        ENUM_ENTRY(v1),                                                                                                        \
        ENUM_ENTRY(v2),                                                                                                        \
        ENUM_ENTRY(v3),                                                                                                        \
        ENUM_ENTRY(v4),                                                                                                        \
        ENUM_ENTRY(v5),                                                                                                        \
        ENUM_ENTRY(v6),                                                                                                        \
        ENUM_ENTRY(v7),                                                                                                        \
        ENUM_ENTRY(v8),                                                                                                        \
        ENUM_ENTRY(v9),                                                                                                        \
        ENUM_ENTRY(v10),                                                                                                       \
        ENUM_ENTRY(v11),                                                                                                       \
        ENUM_ENTRY(v12),                                                                                                       \
        ENUM_ENTRY(v13),                                                                                                       \
        ENUM_ENTRY(v14),                                                                                                       \
        ENUM_ENTRY(v15),                                                                                                       \
        ENUM_ENTRY(v16),                                                                                                       \
        ENUM_ENTRY(v17),                                                                                                       \
        ENUM_ENTRY(v18)                                                                                                        \
    END_ENUM

#define ENUM20(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19)                      \
    BEGIN_ENUM(type, 20, ENUM_LIST(20, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19))) \
        ENUM_ENTRY(v0),                                                                                                             \
        ENUM_ENTRY(v1),                                                                                                             \
        ENUM_ENTRY(v2),                                                                                                             \
        ENUM_ENTRY(v3),                                                                                                             \
        ENUM_ENTRY(v4),                                                                                                             \
        ENUM_ENTRY(v5),                                                                                                             \
        ENUM_ENTRY(v6),                                                                                                             \
        ENUM_ENTRY(v7),                                                                                                             \
        ENUM_ENTRY(v8),                                                                                                             \
        ENUM_ENTRY(v9),                                                                                                             \
        ENUM_ENTRY(v10),                                                                                                            \
        ENUM_ENTRY(v11),                                                                                                            \
        ENUM_ENTRY(v12),                                                                                                            \
        ENUM_ENTRY(v13),                                                                                                            \
        ENUM_ENTRY(v14),                                                                                                            \
        ENUM_ENTRY(v15),                                                                                                            \
        ENUM_ENTRY(v16),                                                                                                            \
        ENUM_ENTRY(v17),                                                                                                            \
        ENUM_ENTRY(v18),                                                                                                            \
        ENUM_ENTRY(v19)                                                                                                             \
    END_ENUM

#define ENUM21(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20)                      \
    BEGIN_ENUM(type, 21, ENUM_LIST(21, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20))) \
        ENUM_ENTRY(v0),                                                                                                                  \
        ENUM_ENTRY(v1),                                                                                                                  \
        ENUM_ENTRY(v2),                                                                                                                  \
        ENUM_ENTRY(v3),                                                                                                                  \
        ENUM_ENTRY(v4),                                                                                                                  \
        ENUM_ENTRY(v5),                                                                                                                  \
        ENUM_ENTRY(v6),                                                                                                                  \
        ENUM_ENTRY(v7),                                                                                                                  \
        ENUM_ENTRY(v8),                                                                                                                  \
        ENUM_ENTRY(v9),                                                                                                                  \
        ENUM_ENTRY(v10),                                                                                                                 \
        ENUM_ENTRY(v11),                                                                                                                 \
        ENUM_ENTRY(v12),                                                                                                                 \
        ENUM_ENTRY(v13),                                                                                                                 \
        ENUM_ENTRY(v14),                                                                                                                 \
        ENUM_ENTRY(v15),                                                                                                                 \
        ENUM_ENTRY(v16),                                                                                                                 \
        ENUM_ENTRY(v17),                                                                                                                 \
        ENUM_ENTRY(v18),                                                                                                                 \
        ENUM_ENTRY(v19),                                                                                                                 \
        ENUM_ENTRY(v20)                                                                                                                  \
    END_ENUM

#define ENUM22(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21)                      \
    BEGIN_ENUM(type, 22, ENUM_LIST(22, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21))) \
        ENUM_ENTRY(v0),                                                                                                                       \
        ENUM_ENTRY(v1),                                                                                                                       \
        ENUM_ENTRY(v2),                                                                                                                       \
        ENUM_ENTRY(v3),                                                                                                                       \
        ENUM_ENTRY(v4),                                                                                                                       \
        ENUM_ENTRY(v5),                                                                                                                       \
        ENUM_ENTRY(v6),                                                                                                                       \
        ENUM_ENTRY(v7),                                                                                                                       \
        ENUM_ENTRY(v8),                                                                                                                       \
        ENUM_ENTRY(v9),                                                                                                                       \
        ENUM_ENTRY(v10),                                                                                                                      \
        ENUM_ENTRY(v11),                                                                                                                      \
        ENUM_ENTRY(v12),                                                                                                                      \
        ENUM_ENTRY(v13),                                                                                                                      \
        ENUM_ENTRY(v14),                                                                                                                      \
        ENUM_ENTRY(v15),                                                                                                                      \
        ENUM_ENTRY(v16),                                                                                                                      \
        ENUM_ENTRY(v17),                                                                                                                      \
        ENUM_ENTRY(v18),                                                                                                                      \
        ENUM_ENTRY(v19),                                                                                                                      \
        ENUM_ENTRY(v20),                                                                                                                      \
        ENUM_ENTRY(v21)                                                                                                                       \
    END_ENUM

#define ENUM23(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22)                      \
    BEGIN_ENUM(type, 23, ENUM_LIST(23, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22))) \
        ENUM_ENTRY(v0),                                                                                                                            \
        ENUM_ENTRY(v1),                                                                                                                            \
        ENUM_ENTRY(v2),                                                                                                                            \
        ENUM_ENTRY(v3),                                                                                                                            \
        ENUM_ENTRY(v4),                                                                                                                            \
        ENUM_ENTRY(v5),                                                                                                                            \
        ENUM_ENTRY(v6),                                                                                                                            \
        ENUM_ENTRY(v7),                                                                                                                            \
        ENUM_ENTRY(v8),                                                                                                                            \
        ENUM_ENTRY(v9),                                                                                                                            \
        ENUM_ENTRY(v10),                                                                                                                           \
        ENUM_ENTRY(v11),                                                                                                                           \
        ENUM_ENTRY(v12),                                                                                                                           \
        ENUM_ENTRY(v13),                                                                                                                           \
        ENUM_ENTRY(v14),                                                                                                                           \
        ENUM_ENTRY(v15),                                                                                                                           \
        ENUM_ENTRY(v16),                                                                                                                           \
        ENUM_ENTRY(v17),                                                                                                                           \
        ENUM_ENTRY(v18),                                                                                                                           \
        ENUM_ENTRY(v19),                                                                                                                           \
        ENUM_ENTRY(v20),                                                                                                                           \
        ENUM_ENTRY(v21),                                                                                                                           \
        ENUM_ENTRY(v22)                                                                                                                            \
    END_ENUM

#define ENUM24(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23)                      \
    BEGIN_ENUM(type, 24, ENUM_LIST(24, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23))) \
        ENUM_ENTRY(v0),                                                                                                                                 \
        ENUM_ENTRY(v1),                                                                                                                                 \
        ENUM_ENTRY(v2),                                                                                                                                 \
        ENUM_ENTRY(v3),                                                                                                                                 \
        ENUM_ENTRY(v4),                                                                                                                                 \
        ENUM_ENTRY(v5),                                                                                                                                 \
        ENUM_ENTRY(v6),                                                                                                                                 \
        ENUM_ENTRY(v7),                                                                                                                                 \
        ENUM_ENTRY(v8),                                                                                                                                 \
        ENUM_ENTRY(v9),                                                                                                                                 \
        ENUM_ENTRY(v10),                                                                                                                                \
        ENUM_ENTRY(v11),                                                                                                                                \
        ENUM_ENTRY(v12),                                                                                                                                \
        ENUM_ENTRY(v13),                                                                                                                                \
        ENUM_ENTRY(v14),                                                                                                                                \
        ENUM_ENTRY(v15),                                                                                                                                \
        ENUM_ENTRY(v16),                                                                                                                                \
        ENUM_ENTRY(v17),                                                                                                                                \
        ENUM_ENTRY(v18),                                                                                                                                \
        ENUM_ENTRY(v19),                                                                                                                                \
        ENUM_ENTRY(v20),                                                                                                                                \
        ENUM_ENTRY(v21),                                                                                                                                \
        ENUM_ENTRY(v22),                                                                                                                                \
        ENUM_ENTRY(v23)                                                                                                                                 \
    END_ENUM

#define ENUM25(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24)                      \
    BEGIN_ENUM(type, 25, ENUM_LIST(25, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24))) \
        ENUM_ENTRY(v0),                                                                                                                                      \
        ENUM_ENTRY(v1),                                                                                                                                      \
        ENUM_ENTRY(v2),                                                                                                                                      \
        ENUM_ENTRY(v3),                                                                                                                                      \
        ENUM_ENTRY(v4),                                                                                                                                      \
        ENUM_ENTRY(v5),                                                                                                                                      \
        ENUM_ENTRY(v6),                                                                                                                                      \
        ENUM_ENTRY(v7),                                                                                                                                      \
        ENUM_ENTRY(v8),                                                                                                                                      \
        ENUM_ENTRY(v9),                                                                                                                                      \
        ENUM_ENTRY(v10),                                                                                                                                     \
        ENUM_ENTRY(v11),                                                                                                                                     \
        ENUM_ENTRY(v12),                                                                                                                                     \
        ENUM_ENTRY(v13),                                                                                                                                     \
        ENUM_ENTRY(v14),                                                                                                                                     \
        ENUM_ENTRY(v15),                                                                                                                                     \
        ENUM_ENTRY(v16),                                                                                                                                     \
        ENUM_ENTRY(v17),                                                                                                                                     \
        ENUM_ENTRY(v18),                                                                                                                                     \
        ENUM_ENTRY(v19),                                                                                                                                     \
        ENUM_ENTRY(v20),                                                                                                                                     \
        ENUM_ENTRY(v21),                                                                                                                                     \
        ENUM_ENTRY(v22),                                                                                                                                     \
        ENUM_ENTRY(v23),                                                                                                                                     \
        ENUM_ENTRY(v24)                                                                                                                                      \
    END_ENUM

#define ENUM26(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25)                      \
    BEGIN_ENUM(type, 26, ENUM_LIST(26, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25))) \
        ENUM_ENTRY(v0),                                                                                                                                           \
        ENUM_ENTRY(v1),                                                                                                                                           \
        ENUM_ENTRY(v2),                                                                                                                                           \
        ENUM_ENTRY(v3),                                                                                                                                           \
        ENUM_ENTRY(v4),                                                                                                                                           \
        ENUM_ENTRY(v5),                                                                                                                                           \
        ENUM_ENTRY(v6),                                                                                                                                           \
        ENUM_ENTRY(v7),                                                                                                                                           \
        ENUM_ENTRY(v8),                                                                                                                                           \
        ENUM_ENTRY(v9),                                                                                                                                           \
        ENUM_ENTRY(v10),                                                                                                                                          \
        ENUM_ENTRY(v11),                                                                                                                                          \
        ENUM_ENTRY(v12),                                                                                                                                          \
        ENUM_ENTRY(v13),                                                                                                                                          \
        ENUM_ENTRY(v14),                                                                                                                                          \
        ENUM_ENTRY(v15),                                                                                                                                          \
        ENUM_ENTRY(v16),                                                                                                                                          \
        ENUM_ENTRY(v17),                                                                                                                                          \
        ENUM_ENTRY(v18),                                                                                                                                          \
        ENUM_ENTRY(v19),                                                                                                                                          \
        ENUM_ENTRY(v20),                                                                                                                                          \
        ENUM_ENTRY(v21),                                                                                                                                          \
        ENUM_ENTRY(v22),                                                                                                                                          \
        ENUM_ENTRY(v23),                                                                                                                                          \
        ENUM_ENTRY(v24),                                                                                                                                          \
        ENUM_ENTRY(v25)                                                                                                                                           \
    END_ENUM

#define ENUM27(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26)                      \
    BEGIN_ENUM(type, 27, ENUM_LIST(27, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26))) \
        ENUM_ENTRY(v0),                                                                                                                                                \
        ENUM_ENTRY(v1),                                                                                                                                                \
        ENUM_ENTRY(v2),                                                                                                                                                \
        ENUM_ENTRY(v3),                                                                                                                                                \
        ENUM_ENTRY(v4),                                                                                                                                                \
        ENUM_ENTRY(v5),                                                                                                                                                \
        ENUM_ENTRY(v6),                                                                                                                                                \
        ENUM_ENTRY(v7),                                                                                                                                                \
        ENUM_ENTRY(v8),                                                                                                                                                \
        ENUM_ENTRY(v9),                                                                                                                                                \
        ENUM_ENTRY(v10),                                                                                                                                               \
        ENUM_ENTRY(v11),                                                                                                                                               \
        ENUM_ENTRY(v12),                                                                                                                                               \
        ENUM_ENTRY(v13),                                                                                                                                               \
        ENUM_ENTRY(v14),                                                                                                                                               \
        ENUM_ENTRY(v15),                                                                                                                                               \
        ENUM_ENTRY(v16),                                                                                                                                               \
        ENUM_ENTRY(v17),                                                                                                                                               \
        ENUM_ENTRY(v18),                                                                                                                                               \
        ENUM_ENTRY(v19),                                                                                                                                               \
        ENUM_ENTRY(v20),                                                                                                                                               \
        ENUM_ENTRY(v21),                                                                                                                                               \
        ENUM_ENTRY(v22),                                                                                                                                               \
        ENUM_ENTRY(v23),                                                                                                                                               \
        ENUM_ENTRY(v24),                                                                                                                                               \
        ENUM_ENTRY(v25),                                                                                                                                               \
        ENUM_ENTRY(v26)                                                                                                                                                \
    END_ENUM

#define ENUM28(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27)                      \
    BEGIN_ENUM(type, 28, ENUM_LIST(28, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27))) \
        ENUM_ENTRY(v0),                                                                                                                                                     \
        ENUM_ENTRY(v1),                                                                                                                                                     \
        ENUM_ENTRY(v2),                                                                                                                                                     \
        ENUM_ENTRY(v3),                                                                                                                                                     \
        ENUM_ENTRY(v4),                                                                                                                                                     \
        ENUM_ENTRY(v5),                                                                                                                                                     \
        ENUM_ENTRY(v6),                                                                                                                                                     \
        ENUM_ENTRY(v7),                                                                                                                                                     \
        ENUM_ENTRY(v8),                                                                                                                                                     \
        ENUM_ENTRY(v9),                                                                                                                                                     \
        ENUM_ENTRY(v10),                                                                                                                                                    \
        ENUM_ENTRY(v11),                                                                                                                                                    \
        ENUM_ENTRY(v12),                                                                                                                                                    \
        ENUM_ENTRY(v13),                                                                                                                                                    \
        ENUM_ENTRY(v14),                                                                                                                                                    \
        ENUM_ENTRY(v15),                                                                                                                                                    \
        ENUM_ENTRY(v16),                                                                                                                                                    \
        ENUM_ENTRY(v17),                                                                                                                                                    \
        ENUM_ENTRY(v18),                                                                                                                                                    \
        ENUM_ENTRY(v19),                                                                                                                                                    \
        ENUM_ENTRY(v20),                                                                                                                                                    \
        ENUM_ENTRY(v21),                                                                                                                                                    \
        ENUM_ENTRY(v22),                                                                                                                                                    \
        ENUM_ENTRY(v23),                                                                                                                                                    \
        ENUM_ENTRY(v24),                                                                                                                                                    \
        ENUM_ENTRY(v25),                                                                                                                                                    \
        ENUM_ENTRY(v26),                                                                                                                                                    \
        ENUM_ENTRY(v27)                                                                                                                                                     \
    END_ENUM

#define ENUM29(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28)                      \
    BEGIN_ENUM(type, 29, ENUM_LIST(29, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28))) \
        ENUM_ENTRY(v0),                                                                                                                                                          \
        ENUM_ENTRY(v1),                                                                                                                                                          \
        ENUM_ENTRY(v2),                                                                                                                                                          \
        ENUM_ENTRY(v3),                                                                                                                                                          \
        ENUM_ENTRY(v4),                                                                                                                                                          \
        ENUM_ENTRY(v5),                                                                                                                                                          \
        ENUM_ENTRY(v6),                                                                                                                                                          \
        ENUM_ENTRY(v7),                                                                                                                                                          \
        ENUM_ENTRY(v8),                                                                                                                                                          \
        ENUM_ENTRY(v9),                                                                                                                                                          \
        ENUM_ENTRY(v10),                                                                                                                                                         \
        ENUM_ENTRY(v11),                                                                                                                                                         \
        ENUM_ENTRY(v12),                                                                                                                                                         \
        ENUM_ENTRY(v13),                                                                                                                                                         \
        ENUM_ENTRY(v14),                                                                                                                                                         \
        ENUM_ENTRY(v15),                                                                                                                                                         \
        ENUM_ENTRY(v16),                                                                                                                                                         \
        ENUM_ENTRY(v17),                                                                                                                                                         \
        ENUM_ENTRY(v18),                                                                                                                                                         \
        ENUM_ENTRY(v19),                                                                                                                                                         \
        ENUM_ENTRY(v20),                                                                                                                                                         \
        ENUM_ENTRY(v21),                                                                                                                                                         \
        ENUM_ENTRY(v22),                                                                                                                                                         \
        ENUM_ENTRY(v23),                                                                                                                                                         \
        ENUM_ENTRY(v24),                                                                                                                                                         \
        ENUM_ENTRY(v25),                                                                                                                                                         \
        ENUM_ENTRY(v26),                                                                                                                                                         \
        ENUM_ENTRY(v27),                                                                                                                                                         \
        ENUM_ENTRY(v28)                                                                                                                                                          \
    END_ENUM

#define ENUM30(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29)                      \
    BEGIN_ENUM(type, 30, ENUM_LIST(30, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29))) \
        ENUM_ENTRY(v0),                                                                                                                                                               \
        ENUM_ENTRY(v1),                                                                                                                                                               \
        ENUM_ENTRY(v2),                                                                                                                                                               \
        ENUM_ENTRY(v3),                                                                                                                                                               \
        ENUM_ENTRY(v4),                                                                                                                                                               \
        ENUM_ENTRY(v5),                                                                                                                                                               \
        ENUM_ENTRY(v6),                                                                                                                                                               \
        ENUM_ENTRY(v7),                                                                                                                                                               \
        ENUM_ENTRY(v8),                                                                                                                                                               \
        ENUM_ENTRY(v9),                                                                                                                                                               \
        ENUM_ENTRY(v10),                                                                                                                                                              \
        ENUM_ENTRY(v11),                                                                                                                                                              \
        ENUM_ENTRY(v12),                                                                                                                                                              \
        ENUM_ENTRY(v13),                                                                                                                                                              \
        ENUM_ENTRY(v14),                                                                                                                                                              \
        ENUM_ENTRY(v15),                                                                                                                                                              \
        ENUM_ENTRY(v16),                                                                                                                                                              \
        ENUM_ENTRY(v17),                                                                                                                                                              \
        ENUM_ENTRY(v18),                                                                                                                                                              \
        ENUM_ENTRY(v19),                                                                                                                                                              \
        ENUM_ENTRY(v20),                                                                                                                                                              \
        ENUM_ENTRY(v21),                                                                                                                                                              \
        ENUM_ENTRY(v22),                                                                                                                                                              \
        ENUM_ENTRY(v23),                                                                                                                                                              \
        ENUM_ENTRY(v24),                                                                                                                                                              \
        ENUM_ENTRY(v25),                                                                                                                                                              \
        ENUM_ENTRY(v26),                                                                                                                                                              \
        ENUM_ENTRY(v27),                                                                                                                                                              \
        ENUM_ENTRY(v28),                                                                                                                                                              \
        ENUM_ENTRY(v29)                                                                                                                                                               \
    END_ENUM

#define ENUM31(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30)                      \
    BEGIN_ENUM(type, 31, ENUM_LIST(31, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30))) \
        ENUM_ENTRY(v0),                                                                                                                                                                    \
        ENUM_ENTRY(v1),                                                                                                                                                                    \
        ENUM_ENTRY(v2),                                                                                                                                                                    \
        ENUM_ENTRY(v3),                                                                                                                                                                    \
        ENUM_ENTRY(v4),                                                                                                                                                                    \
        ENUM_ENTRY(v5),                                                                                                                                                                    \
        ENUM_ENTRY(v6),                                                                                                                                                                    \
        ENUM_ENTRY(v7),                                                                                                                                                                    \
        ENUM_ENTRY(v8),                                                                                                                                                                    \
        ENUM_ENTRY(v9),                                                                                                                                                                    \
        ENUM_ENTRY(v10),                                                                                                                                                                   \
        ENUM_ENTRY(v11),                                                                                                                                                                   \
        ENUM_ENTRY(v12),                                                                                                                                                                   \
        ENUM_ENTRY(v13),                                                                                                                                                                   \
        ENUM_ENTRY(v14),                                                                                                                                                                   \
        ENUM_ENTRY(v15),                                                                                                                                                                   \
        ENUM_ENTRY(v16),                                                                                                                                                                   \
        ENUM_ENTRY(v17),                                                                                                                                                                   \
        ENUM_ENTRY(v18),                                                                                                                                                                   \
        ENUM_ENTRY(v19),                                                                                                                                                                   \
        ENUM_ENTRY(v20),                                                                                                                                                                   \
        ENUM_ENTRY(v21),                                                                                                                                                                   \
        ENUM_ENTRY(v22),                                                                                                                                                                   \
        ENUM_ENTRY(v23),                                                                                                                                                                   \
        ENUM_ENTRY(v24),                                                                                                                                                                   \
        ENUM_ENTRY(v25),                                                                                                                                                                   \
        ENUM_ENTRY(v26),                                                                                                                                                                   \
        ENUM_ENTRY(v27),                                                                                                                                                                   \
        ENUM_ENTRY(v28),                                                                                                                                                                   \
        ENUM_ENTRY(v29),                                                                                                                                                                   \
        ENUM_ENTRY(v30)                                                                                                                                                                    \
    END_ENUM

#define ENUM32(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31)                      \
    BEGIN_ENUM(type, 32, ENUM_LIST(32, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31))) \
        ENUM_ENTRY(v0),                                                                                                                                                                         \
        ENUM_ENTRY(v1),                                                                                                                                                                         \
        ENUM_ENTRY(v2),                                                                                                                                                                         \
        ENUM_ENTRY(v3),                                                                                                                                                                         \
        ENUM_ENTRY(v4),                                                                                                                                                                         \
        ENUM_ENTRY(v5),                                                                                                                                                                         \
        ENUM_ENTRY(v6),                                                                                                                                                                         \
        ENUM_ENTRY(v7),                                                                                                                                                                         \
        ENUM_ENTRY(v8),                                                                                                                                                                         \
        ENUM_ENTRY(v9),                                                                                                                                                                         \
        ENUM_ENTRY(v10),                                                                                                                                                                        \
        ENUM_ENTRY(v11),                                                                                                                                                                        \
        ENUM_ENTRY(v12),                                                                                                                                                                        \
        ENUM_ENTRY(v13),                                                                                                                                                                        \
        ENUM_ENTRY(v14),                                                                                                                                                                        \
        ENUM_ENTRY(v15),                                                                                                                                                                        \
        ENUM_ENTRY(v16),                                                                                                                                                                        \
        ENUM_ENTRY(v17),                                                                                                                                                                        \
        ENUM_ENTRY(v18),                                                                                                                                                                        \
        ENUM_ENTRY(v19),                                                                                                                                                                        \
        ENUM_ENTRY(v20),                                                                                                                                                                        \
        ENUM_ENTRY(v21),                                                                                                                                                                        \
        ENUM_ENTRY(v22),                                                                                                                                                                        \
        ENUM_ENTRY(v23),                                                                                                                                                                        \
        ENUM_ENTRY(v24),                                                                                                                                                                        \
        ENUM_ENTRY(v25),                                                                                                                                                                        \
        ENUM_ENTRY(v26),                                                                                                                                                                        \
        ENUM_ENTRY(v27),                                                                                                                                                                        \
        ENUM_ENTRY(v28),                                                                                                                                                                        \
        ENUM_ENTRY(v29),                                                                                                                                                                        \
        ENUM_ENTRY(v30),                                                                                                                                                                        \
        ENUM_ENTRY(v31)                                                                                                                                                                         \
    END_ENUM

#define ENUM33(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32)                      \
    BEGIN_ENUM(type, 33, ENUM_LIST(33, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32))) \
        ENUM_ENTRY(v0),                                                                                                                                                                              \
        ENUM_ENTRY(v1),                                                                                                                                                                              \
        ENUM_ENTRY(v2),                                                                                                                                                                              \
        ENUM_ENTRY(v3),                                                                                                                                                                              \
        ENUM_ENTRY(v4),                                                                                                                                                                              \
        ENUM_ENTRY(v5),                                                                                                                                                                              \
        ENUM_ENTRY(v6),                                                                                                                                                                              \
        ENUM_ENTRY(v7),                                                                                                                                                                              \
        ENUM_ENTRY(v8),                                                                                                                                                                              \
        ENUM_ENTRY(v9),                                                                                                                                                                              \
        ENUM_ENTRY(v10),                                                                                                                                                                             \
        ENUM_ENTRY(v11),                                                                                                                                                                             \
        ENUM_ENTRY(v12),                                                                                                                                                                             \
        ENUM_ENTRY(v13),                                                                                                                                                                             \
        ENUM_ENTRY(v14),                                                                                                                                                                             \
        ENUM_ENTRY(v15),                                                                                                                                                                             \
        ENUM_ENTRY(v16),                                                                                                                                                                             \
        ENUM_ENTRY(v17),                                                                                                                                                                             \
        ENUM_ENTRY(v18),                                                                                                                                                                             \
        ENUM_ENTRY(v19),                                                                                                                                                                             \
        ENUM_ENTRY(v20),                                                                                                                                                                             \
        ENUM_ENTRY(v21),                                                                                                                                                                             \
        ENUM_ENTRY(v22),                                                                                                                                                                             \
        ENUM_ENTRY(v23),                                                                                                                                                                             \
        ENUM_ENTRY(v24),                                                                                                                                                                             \
        ENUM_ENTRY(v25),                                                                                                                                                                             \
        ENUM_ENTRY(v26),                                                                                                                                                                             \
        ENUM_ENTRY(v27),                                                                                                                                                                             \
        ENUM_ENTRY(v28),                                                                                                                                                                             \
        ENUM_ENTRY(v29),                                                                                                                                                                             \
        ENUM_ENTRY(v30),                                                                                                                                                                             \
        ENUM_ENTRY(v31),                                                                                                                                                                             \
        ENUM_ENTRY(v32)                                                                                                                                                                              \
    END_ENUM

#define ENUM34(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33)                      \
    BEGIN_ENUM(type, 34, ENUM_LIST(34, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                   \
        ENUM_ENTRY(v1),                                                                                                                                                                                   \
        ENUM_ENTRY(v2),                                                                                                                                                                                   \
        ENUM_ENTRY(v3),                                                                                                                                                                                   \
        ENUM_ENTRY(v4),                                                                                                                                                                                   \
        ENUM_ENTRY(v5),                                                                                                                                                                                   \
        ENUM_ENTRY(v6),                                                                                                                                                                                   \
        ENUM_ENTRY(v7),                                                                                                                                                                                   \
        ENUM_ENTRY(v8),                                                                                                                                                                                   \
        ENUM_ENTRY(v9),                                                                                                                                                                                   \
        ENUM_ENTRY(v10),                                                                                                                                                                                  \
        ENUM_ENTRY(v11),                                                                                                                                                                                  \
        ENUM_ENTRY(v12),                                                                                                                                                                                  \
        ENUM_ENTRY(v13),                                                                                                                                                                                  \
        ENUM_ENTRY(v14),                                                                                                                                                                                  \
        ENUM_ENTRY(v15),                                                                                                                                                                                  \
        ENUM_ENTRY(v16),                                                                                                                                                                                  \
        ENUM_ENTRY(v17),                                                                                                                                                                                  \
        ENUM_ENTRY(v18),                                                                                                                                                                                  \
        ENUM_ENTRY(v19),                                                                                                                                                                                  \
        ENUM_ENTRY(v20),                                                                                                                                                                                  \
        ENUM_ENTRY(v21),                                                                                                                                                                                  \
        ENUM_ENTRY(v22),                                                                                                                                                                                  \
        ENUM_ENTRY(v23),                                                                                                                                                                                  \
        ENUM_ENTRY(v24),                                                                                                                                                                                  \
        ENUM_ENTRY(v25),                                                                                                                                                                                  \
        ENUM_ENTRY(v26),                                                                                                                                                                                  \
        ENUM_ENTRY(v27),                                                                                                                                                                                  \
        ENUM_ENTRY(v28),                                                                                                                                                                                  \
        ENUM_ENTRY(v29),                                                                                                                                                                                  \
        ENUM_ENTRY(v30),                                                                                                                                                                                  \
        ENUM_ENTRY(v31),                                                                                                                                                                                  \
        ENUM_ENTRY(v32),                                                                                                                                                                                  \
        ENUM_ENTRY(v33)                                                                                                                                                                                   \
    END_ENUM

#define ENUM35(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34)                      \
    BEGIN_ENUM(type, 35, ENUM_LIST(35, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                        \
        ENUM_ENTRY(v1),                                                                                                                                                                                        \
        ENUM_ENTRY(v2),                                                                                                                                                                                        \
        ENUM_ENTRY(v3),                                                                                                                                                                                        \
        ENUM_ENTRY(v4),                                                                                                                                                                                        \
        ENUM_ENTRY(v5),                                                                                                                                                                                        \
        ENUM_ENTRY(v6),                                                                                                                                                                                        \
        ENUM_ENTRY(v7),                                                                                                                                                                                        \
        ENUM_ENTRY(v8),                                                                                                                                                                                        \
        ENUM_ENTRY(v9),                                                                                                                                                                                        \
        ENUM_ENTRY(v10),                                                                                                                                                                                       \
        ENUM_ENTRY(v11),                                                                                                                                                                                       \
        ENUM_ENTRY(v12),                                                                                                                                                                                       \
        ENUM_ENTRY(v13),                                                                                                                                                                                       \
        ENUM_ENTRY(v14),                                                                                                                                                                                       \
        ENUM_ENTRY(v15),                                                                                                                                                                                       \
        ENUM_ENTRY(v16),                                                                                                                                                                                       \
        ENUM_ENTRY(v17),                                                                                                                                                                                       \
        ENUM_ENTRY(v18),                                                                                                                                                                                       \
        ENUM_ENTRY(v19),                                                                                                                                                                                       \
        ENUM_ENTRY(v20),                                                                                                                                                                                       \
        ENUM_ENTRY(v21),                                                                                                                                                                                       \
        ENUM_ENTRY(v22),                                                                                                                                                                                       \
        ENUM_ENTRY(v23),                                                                                                                                                                                       \
        ENUM_ENTRY(v24),                                                                                                                                                                                       \
        ENUM_ENTRY(v25),                                                                                                                                                                                       \
        ENUM_ENTRY(v26),                                                                                                                                                                                       \
        ENUM_ENTRY(v27),                                                                                                                                                                                       \
        ENUM_ENTRY(v28),                                                                                                                                                                                       \
        ENUM_ENTRY(v29),                                                                                                                                                                                       \
        ENUM_ENTRY(v30),                                                                                                                                                                                       \
        ENUM_ENTRY(v31),                                                                                                                                                                                       \
        ENUM_ENTRY(v32),                                                                                                                                                                                       \
        ENUM_ENTRY(v33),                                                                                                                                                                                       \
        ENUM_ENTRY(v34)                                                                                                                                                                                        \
    END_ENUM

#define ENUM36(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35)                      \
    BEGIN_ENUM(type, 36, ENUM_LIST(36, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                             \
        ENUM_ENTRY(v1),                                                                                                                                                                                             \
        ENUM_ENTRY(v2),                                                                                                                                                                                             \
        ENUM_ENTRY(v3),                                                                                                                                                                                             \
        ENUM_ENTRY(v4),                                                                                                                                                                                             \
        ENUM_ENTRY(v5),                                                                                                                                                                                             \
        ENUM_ENTRY(v6),                                                                                                                                                                                             \
        ENUM_ENTRY(v7),                                                                                                                                                                                             \
        ENUM_ENTRY(v8),                                                                                                                                                                                             \
        ENUM_ENTRY(v9),                                                                                                                                                                                             \
        ENUM_ENTRY(v10),                                                                                                                                                                                            \
        ENUM_ENTRY(v11),                                                                                                                                                                                            \
        ENUM_ENTRY(v12),                                                                                                                                                                                            \
        ENUM_ENTRY(v13),                                                                                                                                                                                            \
        ENUM_ENTRY(v14),                                                                                                                                                                                            \
        ENUM_ENTRY(v15),                                                                                                                                                                                            \
        ENUM_ENTRY(v16),                                                                                                                                                                                            \
        ENUM_ENTRY(v17),                                                                                                                                                                                            \
        ENUM_ENTRY(v18),                                                                                                                                                                                            \
        ENUM_ENTRY(v19),                                                                                                                                                                                            \
        ENUM_ENTRY(v20),                                                                                                                                                                                            \
        ENUM_ENTRY(v21),                                                                                                                                                                                            \
        ENUM_ENTRY(v22),                                                                                                                                                                                            \
        ENUM_ENTRY(v23),                                                                                                                                                                                            \
        ENUM_ENTRY(v24),                                                                                                                                                                                            \
        ENUM_ENTRY(v25),                                                                                                                                                                                            \
        ENUM_ENTRY(v26),                                                                                                                                                                                            \
        ENUM_ENTRY(v27),                                                                                                                                                                                            \
        ENUM_ENTRY(v28),                                                                                                                                                                                            \
        ENUM_ENTRY(v29),                                                                                                                                                                                            \
        ENUM_ENTRY(v30),                                                                                                                                                                                            \
        ENUM_ENTRY(v31),                                                                                                                                                                                            \
        ENUM_ENTRY(v32),                                                                                                                                                                                            \
        ENUM_ENTRY(v33),                                                                                                                                                                                            \
        ENUM_ENTRY(v34),                                                                                                                                                                                            \
        ENUM_ENTRY(v35)                                                                                                                                                                                             \
    END_ENUM

#define ENUM37(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36)                      \
    BEGIN_ENUM(type, 37, ENUM_LIST(37, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                  \
        ENUM_ENTRY(v1),                                                                                                                                                                                                  \
        ENUM_ENTRY(v2),                                                                                                                                                                                                  \
        ENUM_ENTRY(v3),                                                                                                                                                                                                  \
        ENUM_ENTRY(v4),                                                                                                                                                                                                  \
        ENUM_ENTRY(v5),                                                                                                                                                                                                  \
        ENUM_ENTRY(v6),                                                                                                                                                                                                  \
        ENUM_ENTRY(v7),                                                                                                                                                                                                  \
        ENUM_ENTRY(v8),                                                                                                                                                                                                  \
        ENUM_ENTRY(v9),                                                                                                                                                                                                  \
        ENUM_ENTRY(v10),                                                                                                                                                                                                 \
        ENUM_ENTRY(v11),                                                                                                                                                                                                 \
        ENUM_ENTRY(v12),                                                                                                                                                                                                 \
        ENUM_ENTRY(v13),                                                                                                                                                                                                 \
        ENUM_ENTRY(v14),                                                                                                                                                                                                 \
        ENUM_ENTRY(v15),                                                                                                                                                                                                 \
        ENUM_ENTRY(v16),                                                                                                                                                                                                 \
        ENUM_ENTRY(v17),                                                                                                                                                                                                 \
        ENUM_ENTRY(v18),                                                                                                                                                                                                 \
        ENUM_ENTRY(v19),                                                                                                                                                                                                 \
        ENUM_ENTRY(v20),                                                                                                                                                                                                 \
        ENUM_ENTRY(v21),                                                                                                                                                                                                 \
        ENUM_ENTRY(v22),                                                                                                                                                                                                 \
        ENUM_ENTRY(v23),                                                                                                                                                                                                 \
        ENUM_ENTRY(v24),                                                                                                                                                                                                 \
        ENUM_ENTRY(v25),                                                                                                                                                                                                 \
        ENUM_ENTRY(v26),                                                                                                                                                                                                 \
        ENUM_ENTRY(v27),                                                                                                                                                                                                 \
        ENUM_ENTRY(v28),                                                                                                                                                                                                 \
        ENUM_ENTRY(v29),                                                                                                                                                                                                 \
        ENUM_ENTRY(v30),                                                                                                                                                                                                 \
        ENUM_ENTRY(v31),                                                                                                                                                                                                 \
        ENUM_ENTRY(v32),                                                                                                                                                                                                 \
        ENUM_ENTRY(v33),                                                                                                                                                                                                 \
        ENUM_ENTRY(v34),                                                                                                                                                                                                 \
        ENUM_ENTRY(v35),                                                                                                                                                                                                 \
        ENUM_ENTRY(v36)                                                                                                                                                                                                  \
    END_ENUM

#define ENUM38(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37)                      \
    BEGIN_ENUM(type, 38, ENUM_LIST(38, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                       \
        ENUM_ENTRY(v1),                                                                                                                                                                                                       \
        ENUM_ENTRY(v2),                                                                                                                                                                                                       \
        ENUM_ENTRY(v3),                                                                                                                                                                                                       \
        ENUM_ENTRY(v4),                                                                                                                                                                                                       \
        ENUM_ENTRY(v5),                                                                                                                                                                                                       \
        ENUM_ENTRY(v6),                                                                                                                                                                                                       \
        ENUM_ENTRY(v7),                                                                                                                                                                                                       \
        ENUM_ENTRY(v8),                                                                                                                                                                                                       \
        ENUM_ENTRY(v9),                                                                                                                                                                                                       \
        ENUM_ENTRY(v10),                                                                                                                                                                                                      \
        ENUM_ENTRY(v11),                                                                                                                                                                                                      \
        ENUM_ENTRY(v12),                                                                                                                                                                                                      \
        ENUM_ENTRY(v13),                                                                                                                                                                                                      \
        ENUM_ENTRY(v14),                                                                                                                                                                                                      \
        ENUM_ENTRY(v15),                                                                                                                                                                                                      \
        ENUM_ENTRY(v16),                                                                                                                                                                                                      \
        ENUM_ENTRY(v17),                                                                                                                                                                                                      \
        ENUM_ENTRY(v18),                                                                                                                                                                                                      \
        ENUM_ENTRY(v19),                                                                                                                                                                                                      \
        ENUM_ENTRY(v20),                                                                                                                                                                                                      \
        ENUM_ENTRY(v21),                                                                                                                                                                                                      \
        ENUM_ENTRY(v22),                                                                                                                                                                                                      \
        ENUM_ENTRY(v23),                                                                                                                                                                                                      \
        ENUM_ENTRY(v24),                                                                                                                                                                                                      \
        ENUM_ENTRY(v25),                                                                                                                                                                                                      \
        ENUM_ENTRY(v26),                                                                                                                                                                                                      \
        ENUM_ENTRY(v27),                                                                                                                                                                                                      \
        ENUM_ENTRY(v28),                                                                                                                                                                                                      \
        ENUM_ENTRY(v29),                                                                                                                                                                                                      \
        ENUM_ENTRY(v30),                                                                                                                                                                                                      \
        ENUM_ENTRY(v31),                                                                                                                                                                                                      \
        ENUM_ENTRY(v32),                                                                                                                                                                                                      \
        ENUM_ENTRY(v33),                                                                                                                                                                                                      \
        ENUM_ENTRY(v34),                                                                                                                                                                                                      \
        ENUM_ENTRY(v35),                                                                                                                                                                                                      \
        ENUM_ENTRY(v36),                                                                                                                                                                                                      \
        ENUM_ENTRY(v37)                                                                                                                                                                                                       \
    END_ENUM

#define ENUM39(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38)                      \
    BEGIN_ENUM(type, 39, ENUM_LIST(39, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                            \
        ENUM_ENTRY(v1),                                                                                                                                                                                                            \
        ENUM_ENTRY(v2),                                                                                                                                                                                                            \
        ENUM_ENTRY(v3),                                                                                                                                                                                                            \
        ENUM_ENTRY(v4),                                                                                                                                                                                                            \
        ENUM_ENTRY(v5),                                                                                                                                                                                                            \
        ENUM_ENTRY(v6),                                                                                                                                                                                                            \
        ENUM_ENTRY(v7),                                                                                                                                                                                                            \
        ENUM_ENTRY(v8),                                                                                                                                                                                                            \
        ENUM_ENTRY(v9),                                                                                                                                                                                                            \
        ENUM_ENTRY(v10),                                                                                                                                                                                                           \
        ENUM_ENTRY(v11),                                                                                                                                                                                                           \
        ENUM_ENTRY(v12),                                                                                                                                                                                                           \
        ENUM_ENTRY(v13),                                                                                                                                                                                                           \
        ENUM_ENTRY(v14),                                                                                                                                                                                                           \
        ENUM_ENTRY(v15),                                                                                                                                                                                                           \
        ENUM_ENTRY(v16),                                                                                                                                                                                                           \
        ENUM_ENTRY(v17),                                                                                                                                                                                                           \
        ENUM_ENTRY(v18),                                                                                                                                                                                                           \
        ENUM_ENTRY(v19),                                                                                                                                                                                                           \
        ENUM_ENTRY(v20),                                                                                                                                                                                                           \
        ENUM_ENTRY(v21),                                                                                                                                                                                                           \
        ENUM_ENTRY(v22),                                                                                                                                                                                                           \
        ENUM_ENTRY(v23),                                                                                                                                                                                                           \
        ENUM_ENTRY(v24),                                                                                                                                                                                                           \
        ENUM_ENTRY(v25),                                                                                                                                                                                                           \
        ENUM_ENTRY(v26),                                                                                                                                                                                                           \
        ENUM_ENTRY(v27),                                                                                                                                                                                                           \
        ENUM_ENTRY(v28),                                                                                                                                                                                                           \
        ENUM_ENTRY(v29),                                                                                                                                                                                                           \
        ENUM_ENTRY(v30),                                                                                                                                                                                                           \
        ENUM_ENTRY(v31),                                                                                                                                                                                                           \
        ENUM_ENTRY(v32),                                                                                                                                                                                                           \
        ENUM_ENTRY(v33),                                                                                                                                                                                                           \
        ENUM_ENTRY(v34),                                                                                                                                                                                                           \
        ENUM_ENTRY(v35),                                                                                                                                                                                                           \
        ENUM_ENTRY(v36),                                                                                                                                                                                                           \
        ENUM_ENTRY(v37),                                                                                                                                                                                                           \
        ENUM_ENTRY(v38)                                                                                                                                                                                                            \
    END_ENUM

#define ENUM40(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39)                      \
    BEGIN_ENUM(type, 40, ENUM_LIST(40, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                 \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                \
        ENUM_ENTRY(v39)                                                                                                                                                                                                                 \
    END_ENUM

#define ENUM41(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40)                      \
    BEGIN_ENUM(type, 41, ENUM_LIST(41, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                      \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                     \
        ENUM_ENTRY(v40)                                                                                                                                                                                                                      \
    END_ENUM

#define ENUM42(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41)                      \
    BEGIN_ENUM(type, 42, ENUM_LIST(42, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                           \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                          \
        ENUM_ENTRY(v41)                                                                                                                                                                                                                           \
    END_ENUM

#define ENUM43(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42)                      \
    BEGIN_ENUM(type, 43, ENUM_LIST(43, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                               \
        ENUM_ENTRY(v42)                                                                                                                                                                                                                                \
    END_ENUM

#define ENUM44(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43)                      \
    BEGIN_ENUM(type, 44, ENUM_LIST(44, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                     \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v43)                                                                                                                                                                                                                                     \
    END_ENUM

#define ENUM45(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44)                      \
    BEGIN_ENUM(type, 45, ENUM_LIST(45, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                          \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v43),                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v44)                                                                                                                                                                                                                                          \
    END_ENUM

#define ENUM46(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45)                      \
    BEGIN_ENUM(type, 46, ENUM_LIST(46, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                               \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v43),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v44),                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v45)                                                                                                                                                                                                                                               \
    END_ENUM

#define ENUM47(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46)                      \
    BEGIN_ENUM(type, 47, ENUM_LIST(47, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                                    \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v43),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v44),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v45),                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v46)                                                                                                                                                                                                                                                    \
    END_ENUM

#define ENUM48(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46, v47)                      \
    BEGIN_ENUM(type, 48, ENUM_LIST(48, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46, v47))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                                         \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v43),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v44),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v45),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v46),                                                                                                                                                                                                                                                        \
        ENUM_ENTRY(v47)                                                                                                                                                                                                                                                         \
    END_ENUM

#define ENUM49(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46, v47, v48)                      \
    BEGIN_ENUM(type, 49, ENUM_LIST(49, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46, v47, v48))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                                              \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v43),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v44),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v45),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v46),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v47),                                                                                                                                                                                                                                                             \
        ENUM_ENTRY(v48)                                                                                                                                                                                                                                                              \
    END_ENUM

#define ENUM50(type, v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46, v47, v48, v49)                      \
    BEGIN_ENUM(type, 50, ENUM_LIST(50, (v0, v1, v2, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14, v15, v16, v17, v18, v19, v20, v21, v22, v23, v24, v25, v26, v27, v28, v29, v30, v31, v32, v33, v34, v35, v36, v37, v38, v39, v40, v41, v42, v43, v44, v45, v46, v47, v48, v49))) \
        ENUM_ENTRY(v0),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v1),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v2),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v3),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v4),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v5),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v6),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v7),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v8),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v9),                                                                                                                                                                                                                                                                   \
        ENUM_ENTRY(v10),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v11),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v12),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v13),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v14),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v15),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v16),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v17),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v18),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v19),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v20),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v21),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v22),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v23),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v24),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v25),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v26),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v27),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v28),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v29),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v30),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v31),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v32),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v33),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v34),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v35),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v36),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v37),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v38),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v39),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v40),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v41),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v42),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v43),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v44),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v45),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v46),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v47),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v48),                                                                                                                                                                                                                                                                  \
        ENUM_ENTRY(v49)                                                                                                                                                                                                                                                                   \
    END_ENUM

#endif // #ifndef _KEA_ENUM_H_

// end enum.h
