/******************************************************************** 
 * Quantum Hierarchical State Machine declarations (C version)
 * Copyright 2002-2003 by Miro Samek, http://www.quantum-leaps.com 
 * All Rights Reserved.
 *******************************************************************/
#include "qassert.h"
#include "qhsm.h"

DEFINE_THIS_FILE;
                                                   
enum { Q_EMPTY_SIG = 0 };                /* reserved empty signal */

static QEvent const locStdEvt[] = {
   Q_EMPTY_SIG, 
   Q_INIT_SIG,
   Q_ENTRY_SIG,
   Q_EXIT_SIG
};

#define TRIGGER(state_, sig_) \
   (QState)(*(state_))(me, &locStdEvt[sig_])

const char *QHsmGetVersion(void) { 
   return "QHsm 2.3.0";
}

/*.................................................................*/
QHsm *QHsmCtor_(QHsm *me, QPseudoState initial) {
   me->state__ = QHsm_top;
   me->source__ = (QState)initial;
   return me;
}
/*.................................................................*/
void QHsmXtor_(QHsm *me) {
}
/*.................................................................*/
QSTATE QHsm_top(QHsm *me, QEvent const *e) {
   return 0;
}
/*.................................................................*/
void QHsmInit(QHsm *me, QEvent const *e) {
   register QState s;
   REQUIRE(me->state__ == (QState)QHsm_top &&  /* HSM not executed */
           me->source__!= 0);/*we are about to dereference source__*/
   s = me->state__;             /* save me->state__ in a temporary */
   (*(QPseudoState)me->source__)(me, e); /* top-most initial tran. */
                      /* initial transition must go one level deep */
   ASSERT(s == TRIGGER(me->state__, Q_EMPTY_SIG));
   s = me->state__;                        /* update the temporary */
   (*s)(me, &locStdEvt[Q_ENTRY_SIG]);           /* enter the state */
   while ((*s)(me, &locStdEvt[Q_INIT_SIG]) == 0) {/* init handled? */
                      /* initial transition must go one level deep */
      ASSERT(s == TRIGGER(me->state__, Q_EMPTY_SIG));
      s = me->state__;
      (*s)(me, &locStdEvt[Q_ENTRY_SIG]);     /* enter the substate */
   }
}
/*.................................................................*/
void QHsmDispatch(QHsm *me, QEvent const *e) {
   me->source__ = me->state__;
   do {
      me->source__ = (QState)(*me->source__)(me, e);
   } while (me->source__ != 0);
}
/*.................................................................*/
void QHsmTran_(QHsm *me, QState target) {
   QState entry[7], p, q, s, *e, *lca;
   REQUIRE(target != (QState)QHsm_top); /* cannot target top state */
   for (s = me->state__; s != me->source__; ) {
      QState t;
      ASSERT(s != 0);             /* we are about to dereference s */
      t = TRIGGER(s, Q_EXIT_SIG);
      if (t != 0) {/* exit action unhandled, t points to superstate*/
         s = t;
      }
      else {             /* exit action handled, elicit superstate */
         s = TRIGGER(s, Q_EMPTY_SIG);
      }
   }

   *(e = &entry[0]) = 0;
   *(++e) = target;                      /* assume entry to target */

   /* (a) check source == target (transition to self) */
   if (me->source__ == target) {
      TRIGGER(me->source__, Q_EXIT_SIG);            /* exit source */
      goto inLCA;
   }
   /* (b) check source == target->super */
   p = TRIGGER(target, Q_EMPTY_SIG);
   if (me->source__ == p) {
      goto inLCA;
   }
   /* (c) check source->super == target->super (most common) */
   q = TRIGGER(me->source__, Q_EMPTY_SIG);
   if (q == p) {
      TRIGGER(me->source__, Q_EXIT_SIG);            /* exit source */
      goto inLCA;
   }
   /* (d) check me->source->super == target */
   if (q == target) {
      TRIGGER(me->source__, Q_EXIT_SIG);            /* exit source */
      --e;                                 /* do not enter the LCA */
      goto inLCA;
   }
   /* (e) check rest of source == target->super->super... hierarchy*/
   *(++e) = p;
   for (s = TRIGGER(p, Q_EMPTY_SIG); s != 0; 
        s = TRIGGER(s, Q_EMPTY_SIG)) 
   {
      if (me->source__ == s) {
         goto inLCA;
      }
      *(++e) = s;
   }
   TRIGGER(me->source__, Q_EXIT_SIG);               /* exit source */
   /* (f) check rest of source->super == target->super->super... */
   for (lca = e; *lca != 0; --lca) {
      if (q == *lca) {
         e = lca - 1;                      /* do not enter the LCA */
         goto inLCA;
      }
   }
   /* (g) check each me->source__->super->super..for each target...*/
   for (s = q; s != 0; s = TRIGGER(s, Q_EMPTY_SIG)) {
      for (lca = e; *lca != 0; --lca) {
         if (s == *lca) {
            e = lca - 1;                   /* do not enter the LCA */
            goto inLCA;
         }
      }
      TRIGGER(s, Q_EXIT_SIG);                            /* exit s */
   }   
   ASSERT(0);                                     /* malformed HSM */

inLCA:         /* now we are in the LCA of me->source__ and target */
   ASSERT(e < &entry[sizeof(entry)/sizeof(*entry)]); /* entry fits */
   while ((s = *e--) != 0) {/* retrace entry path in reverse order */
      TRIGGER(s, Q_ENTRY_SIG);                          /* enter s */
   }
   me->state__ = target;                   /* update current state */
   while (TRIGGER(target, Q_INIT_SIG) == 0) {
                      /* initial transition must go one level deep */
      ASSERT(target == TRIGGER(me->state__, Q_EMPTY_SIG));
      target = me->state__;
      TRIGGER(target, Q_ENTRY_SIG);                /* enter target */
   }
}
